package core.notevault.data;

import androidx.room.*;
import core.notevault.data.sync.SyncResponse;
import core.notevault.data.sync.SyncStatus;

import java.time.LocalDateTime;
import java.util.List;

@Dao
public interface MusicNoteDAO {

    @Insert
    long insert(MusicNote musicNote);

    @Insert
    void insertNoteSheet(NoteSheet noteSheet);

    @Query("SELECT * FROM music_notes")
    List<MusicNote> getAllNotes();

    @Query("SELECT * FROM note_sheets WHERE musicNoteId = :musicNoteId")
    List<NoteSheet> getNoteSheetsForMusicSong(long musicNoteId);

    @Query("SELECT * FROM concerts WHERE syncStatus = :status")
    List<Concert> getConcertsWithSyncStatus(SyncStatus status);

    @Insert
    void insertConcert(Concert concert);

    @Query("SELECT * FROM concerts")
    List<Concert> getAllConcerts();

    @Delete
    void deleteConcert(Concert concert);

    @Update
    void updateConcert(Concert concert);

    @Delete
    void deleteSong(MusicNote musicNote);

    @Insert
    void insertConcertSong(ConcertSong concertSong);

    @Query("SELECT m.* FROM music_notes m JOIN concert_songs cs ON m.musicNoteId = cs.musicNoteID WHERE cs.id = :concertID")
    List<MusicNote> getAllMusicNotesOfConcert(long concertID);

    @Query("DELETE FROM concert_songs WHERE concertID = :concertID AND musicNoteID = :songID")
    void deleteConcertSong(long songID, int concertID);

    @Query("UPDATE concerts SET serverSpecificConcertID = :serverUUID WHERE id = :concertID")
    @Transaction
    void updateConcertServerUUID(String serverUUID, int concertID);

    // Methode für das batchweise Update der Konzert-ServerUUIDs
    @Query("DELETE FROM concerts WHERE serverSpecificConcertID IN (:serverSpecificIDs)")
    void deleteConcerts(List<String> serverSpecificIDs);

    @Insert
    @Transaction
    void insertSyncResponses(List<SyncResponse> syncResponses);

    @Query("SELECT MAX(lastModified) FROM concerts")
    LocalDateTime getLatestSyncTimestamp();
}
