package core.notevault.ui.gallery.detail;

import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.DividerItemDecoration;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.room.util.ViewInfo;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import core.notevault.data.Concert;
import core.notevault.data.MusicDatabase;
import core.notevault.data.MusicNote;
import core.notevault.data.MusicNoteDAO;
import core.notevault.databinding.FragmentConcertDetailBinding;
import core.notevault.ui.home.NoteSongAdapter;

import java.util.Collections;
import java.util.List;

public class ConcertDetailFragment extends Fragment {

    public interface OnConcertSongsLoadedListener {
        void onSongsLoaded(List<MusicNote> songs);
    }

    private FragmentConcertDetailBinding binding;
    private Concert concert;
    private ConcertDetailViewModel concertDetailViewModel;
    private NoteSongAdapter noteSongAdapter;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        concertDetailViewModel = new ViewModelProvider(this).get(ConcertDetailViewModel.class);

        binding = FragmentConcertDetailBinding.inflate(inflater, container, false);
        int concertId = getArguments().getInt("concertID");
        String concertTitle = getArguments().getString("concertTitle");
        String concertDate = getArguments().getString("concertDate");
        concert = new Concert(concertId, concertTitle, concertDate);

        binding.textConcertDetailsTitle.setText(concertTitle);
        binding.textConcertDetailsDate.setText(concertDate);

        RecyclerView recyclerView = binding.noteRecyclerView;
        noteSongAdapter = new NoteSongAdapter(this::deleteSongFromConcert);
        recyclerView.setAdapter(noteSongAdapter);
        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));

        DividerItemDecoration dividerItemDecoration = new DividerItemDecoration(recyclerView.getContext(), LinearLayoutManager.VERTICAL);
        recyclerView.addItemDecoration(dividerItemDecoration);

        FloatingActionButton fab = binding.addSongConcert;
        fab.setOnClickListener(view -> {
            ConcertSongSelector concertSongSelector = new ConcertSongSelector();

            Bundle bundle = new Bundle();
            bundle.putString("concertTitle", concertTitle);
            bundle.putString("concertDate", concertDate);
            bundle.putInt("concertID", concertId);

            concertSongSelector.setArguments(bundle);

            concertSongSelector.show(getParentFragmentManager(), ConcertSongSelector.TAG);
        });

        concertDetailViewModel.getConcertSongs().observe(getViewLifecycleOwner(), songs -> {
            noteSongAdapter.updateSongTitles(songs);
        });

        new LoadConcertSongsTask(concertId, songs -> {
            this.concertDetailViewModel.setConcertSongs(songs);
        }).execute();

        return binding.getRoot();
    }

    private void deleteSongFromConcert(MusicNote musicNote) {
        this.concertDetailViewModel.deleteConcertSong(musicNote);
        new Thread(() -> {
            MusicNoteDAO musicNoteDAO = MusicDatabase.getDatabase(getContext()).musicNoteDao();
            musicNoteDAO.deleteConcertSong(musicNote.getMusicNoteId(), concert.getId());
        }).start();
    }

    private class LoadConcertSongsTask extends AsyncTask<Void, Void, List<MusicNote>> {
        private final int concertID;
        private final OnConcertSongsLoadedListener listener;

        public LoadConcertSongsTask(int concertID, OnConcertSongsLoadedListener listener) {
            this.concertID = concertID;
            this.listener = listener;
        }

        @Override
        protected List<MusicNote> doInBackground(Void... voids) {
            MusicDatabase db = MusicDatabase.getDatabase(getContext());
            return db.musicNoteDao().getAllMusicNotesOfConcert(concertID);
        }

        @Override
        protected void onPostExecute(List<MusicNote> songs) {
            Log.d("ConcertDetailFragment", "Loaded Concertsongs: " + songs.size());
            listener.onSongsLoaded(songs);
        }
    }
}
