package core.notevault.ui.gallery.detail;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.SearchView;
import android.widget.TextView;
import androidx.annotation.NonNull;
import androidx.fragment.app.DialogFragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import core.notevault.R;
import core.notevault.data.ConcertSong;
import core.notevault.data.MusicDatabase;
import core.notevault.data.MusicNote;
import core.notevault.data.MusicNoteDAO;
import core.notevault.ui.metadatadialog.MetaDataDialog;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class ConcertSongSelector extends DialogFragment {

    private ConcertSongSelectorAdapter adapter;
    private OnSongSelectedListener listener;
    private int concertID;

    public interface OnSongSelectedListener {
        void onSongsSelected(List<MusicNote> songs, int concertID);
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        LayoutInflater inflater = getActivity().getLayoutInflater();
        View dialogView = inflater.inflate(R.layout.dialog_concert_song_selector, null);

        String concertTitle = getArguments().getString("concertTitle");
        String concertDate = getArguments().getString("concertDate");
        concertID = getArguments().getInt("concertID");

        TextView dialogTitle = dialogView.findViewById(R.id.dialog_title_concert_song_selector);
        dialogTitle.setText("Wähle Stücke für das Konzert '" + concertTitle + "' am " + concertDate + " aus");

        RecyclerView recyclerView = dialogView.findViewById(R.id.concert_song_selector_recycler);
        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));


        adapter = new ConcertSongSelectorAdapter(new ArrayList<>());
        recyclerView.setAdapter(adapter);

        SearchView searchView = dialogView.findViewById(R.id.search_view);
        searchView.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            @Override
            public boolean onQueryTextSubmit(String s) {
                return false;
            }

            @Override
            public boolean onQueryTextChange(String s) {
                adapter.filter(s);
                return true;
            }
        });

        fetchAllSongs();

        return new AlertDialog.Builder(requireContext())
                .setView(dialogView)
                .setPositiveButton("Speichern", (dialog, which) -> {
                    List<MusicNote> selectedSongs = adapter.getSelectedSongs();
                    listener.onSongsSelected(selectedSongs, concertID);
                })
                .setNegativeButton("Abbrechen", (dialog, which) -> {} )
                .create();
    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        if (context instanceof ConcertSongSelector.OnSongSelectedListener) {
            listener = (ConcertSongSelector.OnSongSelectedListener) context;
        } else {
            throw new RuntimeException(context.toString()
                    + " must implement OnMetadataListener");
        }
    }

    private void fetchAllSongs() {
        // Ersetze dies durch das Laden der Songs aus der Datenbank oder einer anderen Quelle
        new LoadSongTitlesTask().execute();
    }

    private class LoadSongTitlesTask extends AsyncTask<Void, Void, List<MusicNote>> {

        @Override
        protected List<MusicNote> doInBackground(Void... voids) {
            MusicDatabase db = MusicDatabase.getDatabase(getContext());
            MusicNoteDAO musicNoteDAO = db.musicNoteDao();
            return musicNoteDAO.getAllNotes();
        }

        @Override
        protected void onPostExecute(List<MusicNote> songs) {
            adapter.setSongs(songs);
        }
    }

    public static String TAG = "ConcertSongSelector";
}
